# -*- coding: utf-8 -*-
"""
Created on 2020/4/22 12:08 AM
---------
@summary:
---------
@author: Boris
@email: boris@bzkj.tech
"""

__all__ = ["SingleSpider", "Spider", "BatchSpider"]

from spider.core.spiders.single_spider import SingleSpider
from spider.core.spiders.spider import Spider
from spider.core.spiders.batch_spider import BatchSpider
